/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.junit.rules.webserver.servlet;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;

/**
 * An HTTP servlet to get a file
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class FileServlet extends AbstractHttpServlet {

    private static final long serialVersionUID = 951086086224696702L;

    /**
     * The file to return as HTTP response
     */
    private final File fileToServe;

    /**
     * The content type of the HTTP response
     */
    private final String contentType;

    /**
     * @param fileToServe
     *            The file to return as HTTP response
     * @param contentType
     *            The content type of the HTTP response. If <code>null</code>, no content-type will be set on the HTTP
     *            response
     */
    public FileServlet(final File fileToServe, final String contentType) {
        this.fileToServe = fileToServe;
        this.contentType = contentType;
    }

    @Override
    public String getPath() {
        return "/" + fileToServe.getName();
    }

    @Override
    protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException,
            IOException {

        this.setsHeaderAndDateHeaderInResponse(resp);
        if (this.contentType != null) {
            resp.setContentType(this.contentType);
        }

        this.createOutputStream(resp);
    }

    private void createOutputStream(final HttpServletResponse resp) throws IOException, ServletException {
        final OutputStream output = resp.getOutputStream();
        try {
            final InputStream inputStream = new FileInputStream(this.fileToServe);
            try {
                IOUtils.copy(inputStream, output);
            } finally {
                inputStream.close();
            }
        } finally {
            output.close();
        }
    }

}
