/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.junit.rules.ftpserver;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPClientConfig;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.ftpserver.ftplet.Authority;
import org.apache.ftpserver.ftplet.FtpException;
import org.apache.ftpserver.usermanager.impl.BaseUser;
import org.apache.ftpserver.usermanager.impl.WritePermission;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;

/**
 * Unit tests of the JUnit external resource {@link FTPServer}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class FTPServerTest {

    @Rule
    public FTPServer ftpServer = new FTPServer();

    @Rule
    public TemporaryFolder temporaryFolder = new TemporaryFolder();

    @Test
    public void test() throws IOException, FtpException {

        final BaseUser user1 = new BaseUser();
        user1.setName("user1");
        user1.setPassword("test");
        final List<Authority> authorities = new ArrayList<Authority>();
        authorities.add(new WritePermission());
        user1.setAuthorities(authorities);
        final File homeDirectory1 = new File(this.ftpServer.getRootFileSystem(), user1.getName());
        assertTrue(homeDirectory1.mkdirs());
        user1.setHomeDirectory(homeDirectory1.getAbsolutePath());
        this.ftpServer.registerUser(user1);

        // Generate a file that will be put on the FTP server, and get from the FTP server
        final File generatedFile = this.temporaryFolder.newFile();
        final FileOutputStream fosGeneratedFile = new FileOutputStream(generatedFile);
        final PrintWriter pwGeneratedFile = new PrintWriter(fosGeneratedFile);
        try {
            pwGeneratedFile.append("A file for unit test");
        } finally {
            pwGeneratedFile.close();
        }

        final FTPClient ftpClient = new FTPClient();
        ftpClient.connect("localhost", this.ftpServer.getFtpPort());
        assertTrue(FTPReply.isPositiveCompletion(ftpClient.getReplyCode()));

        ftpClient.enterLocalPassiveMode();

        assertTrue(ftpClient.login(user1.getName(), user1.getPassword()));

        assertTrue(ftpClient.changeWorkingDirectory(""));
        ftpClient.configure(new FTPClientConfig());

        final String myFilename = "myFile.txt";
        final FileInputStream fisGeneratedFile = new FileInputStream(generatedFile);
        try {
            assertTrue(ftpClient.storeFile(myFilename, fisGeneratedFile));
        } finally {
            fisGeneratedFile.close();
        }

        final String[] filenames = ftpClient.listNames();
        assertEquals(1, filenames.length);
        assertEquals(myFilename, filenames[0]);

        final File retrievedFile = this.temporaryFolder.newFile();
        final FileOutputStream fosRetrievedFile = new FileOutputStream(retrievedFile);
        try {
            assertTrue(ftpClient.retrieveFile(myFilename, fosRetrievedFile));
        } finally {
            fosRetrievedFile.close();
        }
    }

}
